@@ -12,11 +12,6 @@ modify entries, comments or metadata, etc. or create new po files from scratch.
 :func:`~polib.mofile` convenience functions.
 """
 
-__author__ = 'David Jean Louis <izimobil@gmail.com>'
-__version__ = '1.0.7'
-__all__ = ['pofile', 'POFile', 'POEntry', 'mofile', 'MOFile', 'MOEntry',
-           'default_encoding', 'escape', 'unescape', 'detect_encoding', ]
-
 import array
 import codecs
 import os
@@ -24,16 +19,13 @@ import re
 import struct
 import sys
 import textwrap
+import io
 
-try:
-    import io
-except ImportError:
-    # replacement of io.open() for python < 2.6
-    # we use codecs instead
-    class io(object):
-        @staticmethod
-        def open(fpath, mode='r', encoding=None):
-            return codecs.open(fpath, mode, encoding)
+
+__author__ = 'David Jean Louis <izimobil@gmail.com>'
+__version__ = '1.1.1'
+__all__ = ['pofile', 'POFile', 'POEntry', 'mofile', 'MOFile', 'MOEntry',
+           'default_encoding', 'escape', 'unescape', 'detect_encoding', ]
 
 
 # the default encoding to use when encoding cannot be detected
@@ -101,8 +93,8 @@ def _is_file(filename_or_contents):
         In the latter case, this function will always return False.
     """
     try:
-        return os.path.exists(filename_or_contents)
-    except (ValueError, UnicodeEncodeError):
+        return os.path.isfile(filename_or_contents)
+    except (TypeError, ValueError, UnicodeEncodeError):
         return False
 # }}}
 # function pofile() {{{
@@ -215,8 +207,8 @@ def detect_encoding(file, binary_mode=False):
             mode = 'r'
             rx = rxt
         f = open(file, mode)
-        for l in f.readlines():
-            match = rx.search(l)
+        for line in f.readlines():
+            match = rx.search(line)
             if match:
                 f.close()
                 enc = match.group(1).strip()
@@ -262,13 +254,30 @@ def unescape(st):
         return m  # handles escaped double quote
     return re.sub(r'\\(\\|n|t|r|")', unescape_repl, st)
 # }}}
+# function natural_sort() {{{
+
+
+def natural_sort(lst):
+    """
+    Sort naturally the given list.
+    Credits: http://stackoverflow.com/a/4836734
+    """
+    def convert(text):
+        return int(text) if text.isdigit() else text.lower()
+
+    def alphanum_key(key):
+        return [convert(c) for c in re.split('([0-9]+)', key)]
+
+    return sorted(lst, key=alphanum_key)
+
+# }}}
 # class _BaseFile {{{
 
 
 class _BaseFile(list):
     """
     Common base class for the :class:`~polib.POFile` and :class:`~polib.MOFile`
-    classes. This class should **not** be instanciated directly.
+    classes. This class should **not** be instantiated directly.
     """
 
     def __init__(self, *args, **kwargs):
@@ -321,10 +330,6 @@ class _BaseFile(list):
         for entry in self.obsolete_entries():
             ret.append(entry.__unicode__(self.wrapwidth))
         ret = u('\n').join(ret)
-
-        assert isinstance(ret, text_type)
-        #if type(ret) != text_type:
-        #    return unicode(ret, self.encoding)
         return ret
 
     if PY3:
@@ -367,7 +372,9 @@ class _BaseFile(list):
         ``entry``
             an instance of :class:`~polib._BaseEntry`.
         """
-        if self.check_for_duplicates and entry in self:
+        # check_for_duplicates may not be defined (yet) when unpickling.
+        # But if pickling, we never want to check for duplicates anyway.
+        if getattr(self, 'check_for_duplicates', False) and entry in self:
             raise ValueError('Entry "%s" already exists' % entry.msgid)
         super(_BaseFile, self).append(entry)
 
@@ -405,7 +412,7 @@ class _BaseFile(list):
             e.flags.append('fuzzy')
         return e
 
-    def save(self, fpath=None, repr_method='__unicode__'):
+    def save(self, fpath=None, repr_method='__unicode__', newline=None):
         """
         Saves the po file to ``fpath``.
         If it is an existing file and no ``fpath`` is provided, then the
@@ -418,6 +425,9 @@ class _BaseFile(list):
 
         ``repr_method``
             string, the method to use for output.
+
+        ``newline``
+            string, controls how universal newlines works
         """
         if self.fpath is None and fpath is None:
             raise IOError('You must provide a file path to save() method')
@@ -427,7 +437,12 @@ class _BaseFile(list):
         if repr_method == 'to_binary':
             fhandle = open(fpath, 'wb')
         else:
-            fhandle = io.open(fpath, 'w', encoding=self.encoding)
+            fhandle = io.open(
+                fpath,
+                'w',
+                encoding=self.encoding,
+                newline=newline
+            )
             if not isinstance(contents, text_type):
                 contents = contents.decode(self.encoding)
         fhandle.write(contents)
@@ -461,11 +476,25 @@ class _BaseFile(list):
             entries = self[:]
         else:
             entries = [e for e in self if not e.obsolete]
+        matches = []
         for e in entries:
             if getattr(e, by) == st:
                 if msgctxt is not False and e.msgctxt != msgctxt:
                     continue
-                return e
+                matches.append(e)
+        if len(matches) == 1:
+            return matches[0]
+        elif len(matches) > 1:
+            if not msgctxt:
+                # find the entry with no msgctx
+                e = None
+                for m in matches:
+                    if not m.msgctxt:
+                        e = m
+                if e:
+                    return e
+                # fallback to the first entry found
+                return matches[0]
         return None
 
     def ordered_metadata(self):
@@ -483,10 +512,10 @@ class _BaseFile(list):
             'PO-Revision-Date',
             'Last-Translator',
             'Language-Team',
+            'Language',
             'MIME-Version',
             'Content-Type',
             'Content-Transfer-Encoding',
-            'Language',
             'Plural-Forms'
         ]
         ordered_data = []
@@ -498,7 +527,7 @@ class _BaseFile(list):
                 pass
         # the rest of the metadata will be alphabetically ordered since there
         # are no specs for this AFAIK
-        for data in sorted(metadata.keys()):
+        for data in natural_sort(metadata.keys()):
             value = metadata[data]
             ordered_data.append((data, value))
         return ordered_data
@@ -522,9 +551,8 @@ class _BaseFile(list):
             else:
                 return 0
         # add metadata entry
-        entries.sort(key=lambda o: o.msgctxt or o.msgid)
+        entries.sort(key=lambda o: o.msgid_with_context.encode('utf-8'))
         mentry = self.metadata_as_entry()
-        #mentry.msgstr = mentry.msgstr.replace('\\n', '').lstrip()
         entries = [mentry] + entries
         entries_len = len(entries)
         ids, strs = b(''), b('')
@@ -656,13 +684,13 @@ class POFile(_BaseFile):
         Convenience method that returns the list of untranslated entries.
         """
         return [e for e in self if not e.translated() and not e.obsolete
-                and not 'fuzzy' in e.flags]
+                and not e.fuzzy]
 
     def fuzzy_entries(self):
         """
         Convenience method that returns the list of fuzzy entries.
         """
-        return [e for e in self if 'fuzzy' in e.flags]
+        return [e for e in self if e.fuzzy]
 
     def obsolete_entries(self):
         """
@@ -687,18 +715,20 @@ class POFile(_BaseFile):
             object POFile, the reference catalog.
         """
         # Store entries in dict/set for faster access
-        self_entries = dict((entry.msgid, entry) for entry in self)
-        refpot_msgids = set(entry.msgid for entry in refpot)
+        self_entries = dict(
+            (entry.msgid_with_context, entry) for entry in self
+        )
+        refpot_msgids = set(entry.msgid_with_context for entry in refpot)
         # Merge entries that are in the refpot
         for entry in refpot:
-            e = self_entries.get(entry.msgid)
+            e = self_entries.get(entry.msgid_with_context)
             if e is None:
                 e = POEntry()
                 self.append(e)
             e.merge(entry)
         # ok, now we must "obsolete" entries that are not in the refpot anymore
         for entry in self:
-            if entry.msgid not in refpot_msgids:
+            if entry.msgid_with_context not in refpot_msgids:
                 entry.obsolete = True
 # }}}
 # class MOFile {{{
@@ -780,7 +810,7 @@ class MOFile(_BaseFile):
 class _BaseEntry(object):
     """
     Base class for :class:`~polib.POEntry` and :class:`~polib.MOEntry` classes.
-    This class should **not** be instanciated directly.
+    This class should **not** be instantiated directly.
     """
 
     def __init__(self, *args, **kwargs):
@@ -885,7 +915,7 @@ class _BaseEntry(object):
             real_wrapwidth = wrapwidth - flength + specialchars_count
             if wrapwidth > 0 and len(field) > real_wrapwidth:
                 # Wrap the line but take field name into account
-                lines = [''] + [unescape(item) for item in wrap(
+                lines = [''] + [unescape(item) for item in textwrap.wrap(
                     escaped_field,
                     wrapwidth - 2,  # 2 for quotes ""
                     drop_whitespace=False,
@@ -902,6 +932,12 @@ class _BaseEntry(object):
         for line in lines:
             ret.append('%s"%s"' % (delflag, escape(line)))
         return ret
+
+    @property
+    def msgid_with_context(self):
+        if self.msgctxt:
+            return '%s%s%s' % (self.msgctxt, "\x04", self.msgid)
+        return self.msgid
 # }}}
 # class POEntry {{{
 
@@ -953,18 +989,18 @@ class POEntry(_BaseEntry):
         """
         Returns the unicode representation of the entry.
         """
-        if self.obsolete:
-            return _BaseEntry.__unicode__(self, wrapwidth)
-
         ret = []
         # comments first, if any (with text wrapping as xgettext does)
-        comments = [('comment', '#. '), ('tcomment', '# ')]
+        if self.obsolete:
+            comments = [('tcomment', '# ')]
+        else:
+            comments = [('comment', '#. '), ('tcomment', '# ')]
         for c in comments:
             val = getattr(self, c[0])
             if val:
                 for comment in val.split('\n'):
                     if wrapwidth > 0 and len(comment) + len(c[1]) > wrapwidth:
-                        ret += wrap(
+                        ret += textwrap.wrap(
                             comment,
                             wrapwidth,
                             initial_indent=c[1],
@@ -975,7 +1011,7 @@ class POEntry(_BaseEntry):
                         ret.append('%s%s' % (c[1], comment))
 
         # occurrences (with text wrapping as xgettext does)
-        if self.occurrences:
+        if not self.obsolete and self.occurrences:
             filelist = []
             for fpath, lineno in self.occurrences:
                 if lineno:
@@ -988,7 +1024,7 @@ class POEntry(_BaseEntry):
                 # what we want for filenames, so the dirty hack is to
                 # temporally replace hyphens with a char that a file cannot
                 # contain, like "*"
-                ret += [l.replace('*', '-') for l in wrap(
+                ret += [line.replace('*', '-') for line in textwrap.wrap(
                     filestr.replace('-', '*'),
                     wrapwidth,
                     initial_indent='#: ',
@@ -1005,24 +1041,23 @@ class POEntry(_BaseEntry):
         # previous context and previous msgid/msgid_plural
         fields = ['previous_msgctxt', 'previous_msgid',
                   'previous_msgid_plural']
+        if self.obsolete:
+            prefix = "#~| "
+        else:
+            prefix = "#| "
         for f in fields:
             val = getattr(self, f)
             if val:
-                ret += self._str_field(f, "#| ", "", val, wrapwidth)
+                ret += self._str_field(f, prefix, "", val, wrapwidth)
 
         ret.append(_BaseEntry.__unicode__(self, wrapwidth))
         ret = u('\n').join(ret)
-
-        assert isinstance(ret, text_type)
-        #if type(ret) != types.UnicodeType:
-        #    return unicode(ret, self.encoding)
         return ret
 
     def __cmp__(self, other):
         """
         Called by comparison operations if rich comparison is not defined.
         """
-
         # First: Obsolete test
         if self.obsolete != other.obsolete:
             if self.obsolete:
@@ -1032,39 +1067,41 @@ class POEntry(_BaseEntry):
         # Work on a copy to protect original
         occ1 = sorted(self.occurrences[:])
         occ2 = sorted(other.occurrences[:])
-        pos = 0
-        for entry1 in occ1:
-            try:
-                entry2 = occ2[pos]
-            except IndexError:
-                return 1
-            pos = pos + 1
-            if entry1[0] != entry2[0]:
-                if entry1[0] > entry2[0]:
-                    return 1
-                else:
-                    return -1
-            if entry1[1] != entry2[1]:
-                if entry1[1] > entry2[1]:
-                    return 1
-                else:
-                    return -1
-        # Compare msgid_plural if set
-        if self.msgid_plural:
-            if not other.msgid_plural:
-                return 1
-            for pos in self.msgid_plural:
-                if pos not in other.msgid_plural:
-                    return 1
-                if self.msgid_plural[pos] > other.msgid_plural[pos]:
-                    return 1
-                if self.msgid_plural[pos] < other.msgid_plural[pos]:
-                    return -1
-        # Finally: Compare message ID
+        if occ1 > occ2:
+            return 1
+        if occ1 < occ2:
+            return -1
+        # Compare context
+        msgctxt = self.msgctxt or '0'
+        othermsgctxt = other.msgctxt or '0'
+        if msgctxt > othermsgctxt:
+            return 1
+        elif msgctxt < othermsgctxt:
+            return -1
+        # Compare msgid_plural
+        msgid_plural = self.msgid_plural or '0'
+        othermsgid_plural = other.msgid_plural or '0'
+        if msgid_plural > othermsgid_plural:
+            return 1
+        elif msgid_plural < othermsgid_plural:
+            return -1
+        # Compare msgstr_plural
+        msgstr_plural = self.msgstr_plural or '0'
+        othermsgstr_plural = other.msgstr_plural or '0'
+        if msgstr_plural > othermsgstr_plural:
+            return 1
+        elif msgstr_plural < othermsgstr_plural:
+            return -1
+        # Compare msgid
         if self.msgid > other.msgid:
             return 1
         elif self.msgid < other.msgid:
             return -1
+        # Compare msgstr
+        if self.msgstr > other.msgstr:
+            return 1
+        elif self.msgstr < other.msgstr:
+            return -1
         return 0
 
     def __gt__(self, other):
@@ -1090,7 +1127,7 @@ class POEntry(_BaseEntry):
         Returns ``True`` if the entry has been translated or ``False``
         otherwise.
         """
-        if self.obsolete or 'fuzzy' in self.flags:
+        if self.obsolete or self.fuzzy:
             return False
         if self.msgstr != '':
             return True
@@ -1109,7 +1146,7 @@ class POEntry(_BaseEntry):
         self.msgctxt = other.msgctxt
         self.occurrences = other.occurrences
         self.comment = other.comment
-        fuzzy = 'fuzzy' in self.flags
+        fuzzy = self.fuzzy
         self.flags = other.flags[:]  # clone flags
         if fuzzy:
             self.flags.append('fuzzy')
@@ -1126,6 +1163,10 @@ class POEntry(_BaseEntry):
                 except KeyError:
                     self.msgstr_plural[pos] = ''
 
+    @property
+    def fuzzy(self):
+        return 'fuzzy' in self.flags
+
     def __hash__(self):
         return hash((self.msgid, self.msgstr))
 # }}}
@@ -1274,8 +1315,13 @@ class _POFileParser(object):
             'msgctxt': 'pc',
         }
         tokens = []
+        fpath = '%s ' % self.instance.fpath if self.instance.fpath else ''
         for line in self.fhandle:
             self.current_line += 1
+            if self.current_line == 1:
+                BOM = codecs.BOM_UTF8.decode('utf-8')
+                if line.startswith(BOM):
+                    line = line[len(BOM):]
             line = line.strip()
             if line == '':
                 continue
@@ -1299,9 +1345,9 @@ class _POFileParser(object):
             if tokens[0] in keywords and nb_tokens > 1:
                 line = line[len(tokens[0]):].lstrip()
                 if re.search(r'([^\\]|^)"', line[1:-1]):
-                    raise IOError('Syntax error in po file %s (line %s): '
+                    raise IOError('Syntax error in po file %s(line %s): '
                                   'unescaped double quote found' %
-                                  (self.instance.fpath, self.current_line))
+                                  (fpath, self.current_line))
                 self.current_token = line
                 self.process(keywords[tokens[0]])
                 continue
@@ -1317,9 +1363,9 @@ class _POFileParser(object):
             elif line[:1] == '"':
                 # we are on a continuation line
                 if re.search(r'([^\\]|^)"', line[1:-1]):
-                    raise IOError('Syntax error in po file %s (line %s): '
+                    raise IOError('Syntax error in po file %s(line %s): '
                                   'unescaped double quote found' %
-                                  (self.instance.fpath, self.current_line))
+                                  (fpath, self.current_line))
                 self.process('mc')
 
             elif line[:7] == 'msgstr[':
@@ -1346,8 +1392,8 @@ class _POFileParser(object):
 
             elif tokens[0] == '#|':
                 if nb_tokens <= 1:
-                    raise IOError('Syntax error in po file %s (line %s)' %
-                                  (self.instance.fpath, self.current_line))
+                    raise IOError('Syntax error in po file %s(line %s)' %
+                                  (fpath, self.current_line))
 
                 # Remove the marker and any whitespace right after that.
                 line = line[2:].lstrip()
@@ -1360,16 +1406,16 @@ class _POFileParser(object):
 
                 if nb_tokens == 2:
                     # Invalid continuation line.
-                    raise IOError('Syntax error in po file %s (line %s): '
+                    raise IOError('Syntax error in po file %s(line %s): '
                                   'invalid continuation line' %
-                                  (self.instance.fpath, self.current_line))
+                                  (fpath, self.current_line))
 
                 # we are on a "previous translation" comment line,
                 if tokens[1] not in prev_keywords:
                     # Unknown keyword in previous translation comment.
-                    raise IOError('Syntax error in po file %s (line %s): '
+                    raise IOError('Syntax error in po file %s(line %s): '
                                   'unknown keyword %s' %
-                                  (self.instance.fpath, self.current_line,
+                                  (fpath, self.current_line,
                                    tokens[1]))
 
                 # Remove the keyword and any whitespace
@@ -1379,8 +1425,8 @@ class _POFileParser(object):
                 self.process(prev_keywords[tokens[1]])
 
             else:
-                raise IOError('Syntax error in po file %s (line %s)' %
-                              (self.instance.fpath, self.current_line))
+                raise IOError('Syntax error in po file %s(line %s)' %
+                              (fpath, self.current_line))
 
         if self.current_entry and len(tokens) > 0 and \
            not tokens[0].startswith('#'):
@@ -1490,9 +1536,9 @@ class _POFileParser(object):
         for occurrence in occurrences:
             if occurrence != '':
                 try:
-                    fil, line = occurrence.split(':')
+                    fil, line = occurrence.rsplit(':', 1)
                     if not line.isdigit():
-                        fil = fil + line
+                        fil = occurrence
                         line = ''
                     self.current_entry.occurrences.append((fil, line))
                 except (ValueError, AttributeError):
@@ -1654,7 +1700,7 @@ class _MOFileParser(object):
         version, numofstrings = self._readbinary(ii, 8)
         # from MO file format specs: "A program seeing an unexpected major
         # revision number should stop reading the MO file entirely"
-        if version not in (0, 1):
+        if version >> 16 not in (0, 1):
             raise IOError('Invalid mo file, unexpected major revision number')
         self.instance.version = version
         # original strings and translation strings hash table offset
@@ -1738,98 +1784,3 @@ class _MOFileParser(object):
             return tup[0]
         return tup
 # }}}
-# class TextWrapper {{{
-
-
-class TextWrapper(textwrap.TextWrapper):
-    """
-    Subclass of textwrap.TextWrapper that backport the
-    drop_whitespace option.
-    """
-    def __init__(self, *args, **kwargs):
-        drop_whitespace = kwargs.pop('drop_whitespace', True)
-        textwrap.TextWrapper.__init__(self, *args, **kwargs)
-        self.drop_whitespace = drop_whitespace
-
-    def _wrap_chunks(self, chunks):
-        """_wrap_chunks(chunks : [string]) -> [string]
-
-        Wrap a sequence of text chunks and return a list of lines of
-        length 'self.width' or less.  (If 'break_long_words' is false,
-        some lines may be longer than this.)  Chunks correspond roughly
-        to words and the whitespace between them: each chunk is
-        indivisible (modulo 'break_long_words'), but a line break can
-        come between any two chunks.  Chunks should not have internal
-        whitespace; ie. a chunk is either all whitespace or a "word".
-        Whitespace chunks will be removed from the beginning and end of
-        lines, but apart from that whitespace is preserved.
-        """
-        lines = []
-        if self.width <= 0:
-            raise ValueError("invalid width %r (must be > 0)" % self.width)
-
-        # Arrange in reverse order so items can be efficiently popped
-        # from a stack of chucks.
-        chunks.reverse()
-
-        while chunks:
-
-            # Start the list of chunks that will make up the current line.
-            # cur_len is just the length of all the chunks in cur_line.
-            cur_line = []
-            cur_len = 0
-
-            # Figure out which static string will prefix this line.
-            if lines:
-                indent = self.subsequent_indent
-            else:
-                indent = self.initial_indent
-
-            # Maximum width for this line.
-            width = self.width - len(indent)
-
-            # First chunk on line is whitespace -- drop it, unless this
-            # is the very beginning of the text (ie. no lines started yet).
-            if self.drop_whitespace and chunks[-1].strip() == '' and lines:
-                del chunks[-1]
-
-            while chunks:
-                l = len(chunks[-1])
-
-                # Can at least squeeze this chunk onto the current line.
-                if cur_len + l <= width:
-                    cur_line.append(chunks.pop())
-                    cur_len += l
-
-                # Nope, this line is full.
-                else:
-                    break
-
-            # The current line is full, and the next chunk is too big to
-            # fit on *any* line (not just this one).
-            if chunks and len(chunks[-1]) > width:
-                self._handle_long_word(chunks, cur_line, cur_len, width)
-
-            # If the last chunk on this line is all whitespace, drop it.
-            if self.drop_whitespace and cur_line and not cur_line[-1].strip():
-                del cur_line[-1]
-
-            # Convert current line back to a string and store it in list
-            # of all lines (return value).
-            if cur_line:
-                lines.append(indent + ''.join(cur_line))
-
-        return lines
-# }}}
-# function wrap() {{{
-
-
-def wrap(text, width=70, **kwargs):
-    """
-    Wrap a single paragraph of text, returning a list of wrapped lines.
-    """
-    if sys.version_info < (2, 6):
-        return TextWrapper(width=width, **kwargs).wrap(text)
-    return textwrap.wrap(text, width=width, **kwargs)
-
-# }}}
