@@ -18,7 +18,7 @@ class TestFunctions(unittest.TestCase):
 
     def test_pofile_and_mofile1(self):
         """
-        Test bad usage of pofile/mofile. 
+        Test bad usage of pofile/mofile.
         """
         data = u('''# test for pofile/mofile with string buffer
 msgid ""
@@ -97,6 +97,28 @@ msgstr "bar"
         po = polib.pofile('tests/test_obsolete_previousmsgid.po')
         self.assertTrue(isinstance(po, polib.POFile))
 
+    def test_ufeff_data_pofile(self):
+        """
+        Test that an ufeff prefixed pofile returns a POFile instance.
+        """
+        data = u('''\ufeff# test for pofile/mofile with ufeff
+msgid ""
+msgstr ""
+"Project-Id-Version: django\n"
+
+msgid "foo"
+msgstr "bar"
+''')
+        po = polib.pofile(data)
+        self.assertTrue(isinstance(po, polib.POFile))
+
+    def test_ufeff_pofile(self):
+        """
+        Test that an ufeff prefixed pofile returns a POFile instance.
+        """
+        po = polib.pofile('tests/test_ufeff.po')
+        self.assertTrue(isinstance(po, polib.POFile))
+
     def test_previous_msgid_1(self):
         """
         Test previous msgid multiline.
@@ -206,7 +228,7 @@ msgstr ""
             exc = sys.exc_info()[1]
             msg = 'Syntax error in po file (line 4): unescaped double quote found'
             self.assertEqual(str(exc), msg)
-    
+
     def test_detect_encoding1(self):
         """
         Test that given encoding is returned when file has no encoding defined.
@@ -238,7 +260,7 @@ msgstr ""
         try:
             self.assertEqual(polib.detect_encoding(data), 'UTF-8')
         finally:
-            f.close()    
+            f.close()
 
     def test_detect_encoding5(self):
         """
@@ -275,26 +297,26 @@ msgstr ""
             polib.unescape('\\\\t and \\\\n and \\\\r and \\\\" and \\\\\\\\'),
             '\\t and \\n and \\r and \\" and \\\\'
         )
-        
+
     def test_pofile_with_subclass(self):
         """
-        Test that the pofile function correctly returns an instance of the 
+        Test that the pofile function correctly returns an instance of the
         passed in class
         """
         class CustomPOFile(polib.POFile):
             pass
-        
+
         pofile = polib.pofile('tests/test_indented.po', klass=CustomPOFile)
         self.assertEqual(pofile.__class__, CustomPOFile)
-        
+
     def test_mofile_with_subclass(self):
         """
-        Test that the mofile function correctly returns an instance of the 
+        Test that the mofile function correctly returns an instance of the
         passed in class
         """
         class CustomMOFile(polib.MOFile):
             pass
-        
+
         mofile = polib.mofile('tests/test_utf8.mo', klass=CustomMOFile)
         self.assertEqual(mofile.__class__, CustomMOFile)
 
@@ -318,6 +340,11 @@ msgstr ""
         po = polib.pofile('tests/test_weird_occurrences.po')
         self.assertEqual(po[0].occurrences, [('C:\\foo\\bar.py', '12')])
 
+    def test_is_file(self):
+        self.assertTrue(polib._is_file(os.path.abspath(__file__)))
+        self.assertFalse(polib._is_file('This is not a file !!!!'))
+        self.assertFalse(polib._is_file(True))
+
 
 class TestBaseFile(unittest.TestCase):
     """
@@ -620,6 +647,62 @@ class TestPoFile(unittest.TestCase):
         e = po.find("Whatever", by='msgid')
         self.assertTrue(isinstance(e, polib.POEntry))
 
+    def test_compare1(self):
+        entry = polib.POEntry(msgid="foo")
+        other = polib.POEntry(msgctxt="Some context", msgid="foo")
+        self.assertNotEqual(entry, other)
+
+    def test_compare2(self):
+        entry = polib.POEntry(msgid="foo")
+        other = polib.POEntry(msgid_plural="Plural", msgid="foo")
+        self.assertNotEqual(entry, other)
+
+    def test_compare3(self):
+        entry = polib.POEntry(
+            msgid_plural="Plural", msgid="foo"
+        )
+        other = polib.POEntry(
+            msgid_plural="Plural", msgstr_plural="Pluriel", msgid="foo"
+        )
+        self.assertNotEqual(entry, other)
+
+    def test_compare4(self):
+        entry = polib.POEntry(msgctxt="Some context", msgid="foo")
+        other = polib.POEntry(msgctxt="Other context", msgid="foo")
+        self.assertNotEqual(entry, other)
+
+    def test_compare5(self):
+        entry = polib.POEntry(msgid_plural="Some plural", msgid="foo")
+        other = polib.POEntry(msgid_plural="Other plural", msgid="foo")
+        self.assertNotEqual(entry, other)
+
+    def test_compare6(self):
+        entry = polib.POEntry(msgstr_plural="Some plural", msgid="foo")
+        other = polib.POEntry(msgstr_plural="Other plural", msgid="foo")
+        self.assertTrue(entry > other)
+
+    def test_compare7(self):
+        entry = polib.POEntry(msgstr="Some msgstr", msgid="foo")
+        other = polib.POEntry(msgstr="Other msgstr", msgid="foo")
+        self.assertTrue(entry >= other)
+        self.assertNotEqual(entry, other)
+        entry = polib.POEntry(msgstr="A msgstr", msgid="foo")
+        other = polib.POEntry(msgstr="Other msgstr", msgid="foo")
+        self.assertTrue(entry <= other)
+        self.assertNotEqual(entry, other)
+
+    def test_compare8(self):
+        entry = polib.POEntry(
+            msgctxt="Some context", msgid_plural="Plural",
+            msgstr_plural="Pluriel", msgid="foo"
+        )
+        other = polib.POEntry(
+            msgctxt="Some context", msgid_plural="Plural",
+            msgstr_plural="Pluriel", msgid="foo"
+        )
+        self.assertEqual(entry, other)
+
+
 class TestMoFile(unittest.TestCase):
     """
     Tests for MoFile class.
@@ -659,7 +742,7 @@ class TestMoFile(unittest.TestCase):
             self.assertEqual(s1, s2)
         finally:
             os.remove(tmpfile)
-        
+
     def test_msgctxt(self):
         #import pdb; pdb.set_trace()
         mo = polib.mofile('tests/test_msgctxt.mo')
@@ -699,25 +782,5 @@ msgstr "oof"
         self.assertEqual(mo.__unicode__(), expected)
 
 
-class TestTextWrap(unittest.TestCase):
-
-    def test_wrap1(self):
-        text = '  Some line that is longer than fifteen characters (whitespace will not be preserved)  '
-        ret = polib.TextWrapper(width=15).wrap(text)
-        expected = [
-            '  Some line', 'that is longer', 'than fifteen', 'characters',
-            '(whitespace', 'will not be', 'preserved)'
-        ]
-        self.assertEqual(ret, expected)
-
-    def test_wrap2(self):
-        text = '  Some line that is longer than fifteen characters (whitespace will be preserved)  '
-        ret = polib.TextWrapper(width=15, drop_whitespace=False).wrap(text)
-        expected = [
-            '  Some line ', 'that is longer ', 'than fifteen ', 'characters ',
-            '(whitespace ', 'will be ', 'preserved)  '
-        ]
-        self.assertEqual(ret, expected)
-
 if __name__ == '__main__':
     unittest.main()
